﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace _01_LinqDemo
{
    static class Program
    {
        static void Main(string[] args)
        {
            // DEMO 1: LINQ Enumarable Query operators
            // Extension methods, Local Type inference, operator chaining
            string[] names = { "Thomas", "Dick", "Harry" , "Marcel", "Charly"};

            // local type inference and standard where operator
            var result = System.Linq.Enumerable.Where(names, (name) => name.Length > 5);
            printResults(result);

            var result1 = names.Where((name) => name.Length > 5);
            printResults(result1);
            
            var resultOrdered = names.OrderBy((name) => name.Length);
            printResults(resultOrdered);


            // extension methods (Custom and .NET FX)
            var result2 = names.EvenLengthNames();
            printResults(result2);

            
            // operator chaining
            var result4 = names
                .Where((name) => name.Length > 5)
                .OrderBy(name => name)
                .Skip(1)
                .Take(2);
            
            printResults(result4);

        }


        static void printResults<T>(IEnumerable<T> results)
        {
            Console.WriteLine("-->");
            foreach (var result in results)
            {
                Console.WriteLine(result);
            }
            Console.WriteLine("<--");
        }

        //custom extension method to provide a EvenLengthNames operator
        public static IEnumerable<T> EvenLengthNames<T>(this IEnumerable<T> inputSequence)
        {

            foreach(var item in inputSequence)
            {
                if(item.ToString().Length %2 ==0 )
                {
                    yield return item;
                }
            }
        }
    }
}
