﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace _02_LinqDemo
{
    static class Program
    {
        static void Main(string[] args)
        {
           // ComprehensionSyntax();
            //DeferedExecution();
           // OuterVariables();
            ShowJoins();
        }

        private static void ComprehensionSyntax()
        {
            // Demo 2: Comprehension Syntax, Deferred Execution, Outer Variable, Some additional queries
            List<String> names = new List<string>() { "Thomas", "Dick", "Harry", "Marcel", "Charly" };
            var extensionmethodsresult = names.Where(name => name.Length > 5)
                                                                .Select(name => name.ToUpper());
            printResults(extensionmethodsresult);

            // Comprehension syntax
            var result = from name in names
                         where name.Length > 5      //names.Where(name => name.Length>5)
                         select name.ToUpper();     //.select(name => name.ToUpper())

            printResults(result);
            
        }
        private static void DeferedExecution()
        {
            // Defered Execution
            List<String> names = new List<string>() { "Thomas", "Dick", "Harry", "Marcel", "Charly" };
            
            // query constructed
            var result2 = from name in names
                          where name.StartsWith("M")
                          select name;
            // you can still tweak the containing sequence
            names.Add("Mary");
            printResults(result2);
            
            // Here you see query get's executed on each enumeration!
            names.Remove("Marcel");
            printResults(result2);
        }

        private static void OuterVariables()
        {
            // Outer Variable
            char[] vowels = { 'a', 'e', 'o', 'i', 'u' };
            IEnumerable<char> query = "not what you might expect";
            query = query.Where(c=>c != 'a');
            query = query.Where(c=>c != 'e');
            query = query.Where(c=>c != 'o');
            query = query.Where(c=>c != 'i');
            query = query.Where(c=>c != 'u');
            printResults(query);

            //reset the query
            query = "not what you might expect";
            foreach (var vowel in vowels)
            {
                query = query.Where(character => character != vowel);
            }

            printResults(query);

            #region solution
            query = "not what you might expect";
            foreach (var vowel in vowels)
            {
                char tempVowel = vowel;
                query = query.Where(character => character != tempVowel);
            }

            printResults(query);
            #endregion
        }

        
        private static void ShowJoins()
        {
            // Show Join operation
            List<Customer> customers = new List<Customer>() { 
                new Customer(){Name="Thomas", itemsPurchased =5, isActive=true},
                new Customer(){Name="Dick", itemsPurchased=3,isActive=true},
                new Customer(){Name="Harry", itemsPurchased=7,isActive=false},
                new Customer(){Name="Marcel", itemsPurchased=9,isActive=true},
                new Customer(){Name="Charly" ,itemsPurchased=7,isActive=false}
            };

            List<Person> persons = new List<Person>() { 
                new Person(){Name="Thomas", Age=27},
                new Person(){Name="Dick", Age=35},
                new Person(){Name="Harry", Age=37},
                new Person(){Name="Marcel", Age=39},
                new Person(){Name="Charly" ,Age=47}
            };

            // inner join on name and if customer is active
            var joinedList = from customer in customers
                             join person in persons on new { name = customer.Name, active = customer.isActive } equals new { name = person.Name, active = true }
                             where person.Age < 40
                             where customer.itemsPurchased > 3
                             select new { name = person.Name, age = person.Age, items = customer.itemsPurchased };

            printResults(joinedList);

        }


        static void printResults<T>(IEnumerable<T> results)
        {
            Console.WriteLine("<----------------->");
            foreach (var result in results)
            {
                if (result.GetType() == typeof(char))
                {
                    Console.Write(result);
                }
                else
                {
                    Console.WriteLine(result);
                }
            }
            Console.WriteLine("\n<----------------->");
        }
    }
}
