﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using System.Diagnostics;

namespace _04_LinqDemo
{
    class Program
    {
        static void Main(string[] args)
        {
            // select all employees living in Arnhem
            XElement xmlEmployees = XElement.Load("employees.xml");

            var employeesLivingInArnhem = from employee in xmlEmployees.Elements("Employee")
                                          where employee.Element("CITY").Value == "ARNHEM"
                                          select new
                                          {
                                              name = employee.Element("NAME").Value,
                                              city = employee.Element("CITY").Value
                                          };

            printResults(employeesLivingInArnhem);

            // get me a count of all the employees we have
            int numberOfEmpoyees = (from employee in xmlEmployees.Elements("Employee")
                                    select employee).Count();

            Console.WriteLine("Number of Employees: {0}", numberOfEmpoyees);

            // get a list of how many people live in a certain city
            // order by the number of people living there
            var orderedByNumberOfEmployeesFromCity = from employee in xmlEmployees.Elements("Employee")
                                                     group employee by employee.Element("CITY").Value into orderedByCity
                                                     orderby orderedByCity.Count() descending
                                                     select orderedByCity;

            foreach (var emps in orderedByNumberOfEmployeesFromCity)
            {
                Console.WriteLine("City: {0}", emps.Key);
                foreach (var emp in emps)
                {
                    Console.WriteLine("\t\t{0}",emp.Element("NAME").Value);
                }
            }


            // Transform the file , make elements attributes
            XElement newDoc =
                new XElement("Employees",
                from employee in xmlEmployees.Elements("Employee")
                select
                     new XElement("Employee",
                                     new XAttribute("CITY", employee.Element("CITY").Value),
                                     new XAttribute("Name", employee.Element("NAME").Value)
                                 )
                             );
            Console.Write(newDoc.ToString());
        }

        static void printResults<T>(IEnumerable<T> results)
        {
            Console.WriteLine("Resultset starts here -->");
            foreach (var result in results)
            {
                Console.WriteLine(result);
            }
            Console.WriteLine("<-- Resultset ends here");
        }
    }
}
