﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace _06_LinqDemo
{
    class Program
    {
        static void Main(string[] args)
        {
            var dbcontext = new pubsEntities();
            // retireve entities from the database
            var authorList = (from a in dbcontext.authors
                             where a.au_fname.StartsWith("a")
                             orderby a.au_fname
                             select a).Skip(2).Take(3);

            // LINQ2Entities uses the IQuerable interface, meaning it uses the Intepreted query option
            // this means that once we start iterating over the entities we will execute the query
            // this will be transslated into SQL statements and those will be returned.
            // show SQL syntax using Query Profiler or intellitrace window
             printResults(authorList);
            
            // update entities
            author newauthor = new author()
            {
                au_id = "000-00-0000",
                 au_fname ="Marcel",
                 au_lname = "de Vries",
                 address ="Kerkhofweg 12",
                 city = "Warnsveld",
                phone = "408 496-7223",
                zip = "94025"
            };

            dbcontext.authors.AddObject(newauthor);
            dbcontext.SaveChanges();

            // select author back from database
            var authorMarcel = (from a in dbcontext.authors
                                where a.au_fname == "Marcel"
                                select a).FirstOrDefault();

            printResults(new List<author>(){authorMarcel});
           
            // delete the author from the DB
            
            dbcontext.authors.DeleteObject(newauthor);
            dbcontext.SaveChanges();
            
            
        }


        static void printResults(IEnumerable<author> results)
        {


            Console.WriteLine("Resultset starts here -->");
            foreach (var result in results)//- this triggers the SQL transformation and query execution
            {
                Console.WriteLine(result.au_fname);
            }
            Console.WriteLine("<-- Resultset ends here");
        }
    }
}
